!+ 
! Module: lcplot_mod
! 
! Authors:  Nicolas Veyrier - CESR
!           Didier Barret -  CESR
!           Jean-Francois Olive -  CESR
!           Natalie Webb - IRAP         
!
! Description: PLOTS TIME SERIES AND PERFORMS VARIABILITY TESTS
! 
! Comments: 
! 
! $Id: module.f90,v 1.5 1998/01/13 08:39:15 gvacanti Exp $
!- 
module lcplot_mod

  use dal
  use param
  use ErrorHandling
  use metatask_mod
  use dss
  use sla

!  use ssclib  
  
  implicit none

  type(TaskInfo) :: info ! type from metatask_mod

  type test
     real(kind=single) :: Stat , Prob
  end type test

  type data_tests
     real(kind=single) :: Mean_Count , Variance
     type(test) :: Chi_Square , K_S
  end type data_tests

  private 

  public lcplot

contains

  !----------------
  subroutine lcplot
  !----------------
  ! Input parameters :
  !   set : Source time series FITS file name
  !   plotdevice : Plot device type
  !   plotfile : Name of output file containing plots
  !   Binsize : Number of bins to gather, default=5
  !   bkgdyscale : Choice to plot background on same y-scale as counts
  ! Dimension variables :
  !   Nlight : Number of light-curve contained in the FITS file (1<Nlight<9)
  !   Nbin : Number of bin in all light-curves
  ! Time variables :
  !   Time_Vect : Vector containing middle of each time interval
  !   Bin_Width
  ! Counts variables :
  !   Net_Rate , Net_Err , Bkg_Rate , Bkg_Err , Net_Counts : Vectors containing
  !   respectively net count rate&error and background count rate&error and net
  !   source count. Their size is Nlight*Nbin
  !   LabelAI variable : Array containing the labels to add on plot
  !   Good_Bin : Vector explaining which bins can be tested
     
    character(len=128) :: Input_File , Output_File , Device_Type
    integer :: Binning_Factor
    real(kind=single) :: Bin_Width
    real(kind=single),dimension(:),pointer :: Time_Vect
    real(kind=single),dimension(:,:),pointer :: Net_Rate, Net_Err 
    real(kind=single),dimension(:,:),pointer :: Bkg_Rate, Bkg_Err
    real(kind=single),dimension(:,:),pointer :: Net_Count, Net_Total
    logical,dimension(:),pointer :: Good_Bin
    character(len=30) :: Content
    character(len=1000),dimension(23) :: LabelAI
    integer :: Nlight , Nbin
    character(len=12) :: backapp, unitc, unitb, Which_Tests
    logical(boolean),dimension(:),pointer  :: gotnullvals
    type( ColumnT ) :: Col_Rate, Col_Backe
    real(kind=single) :: locnt, hicnt, lobkg, hibkg, mean
    character(len=30) :: Stringb, Stringc
    integer :: Lengthb, Lengthc, bbin, cbin

    Input_File = StringParameter("set")
    Device_Type = change_case(StringParameter("plotdevice"),2)
    Output_File = StringParameter("plotfile")
    Binning_Factor = IntParameter("binsize")
    Which_Tests = StringParameter("tests")

    if (.not. (trim(Which_Tests) == 'both' .or. trim(Which_Tests) == 'chi' .or. trim(Which_Tests) == 'ks')) &
      call error('LCPLOT','TestTypeUnknown')


    call read_file (Input_File, Binning_Factor, Bin_Width, Time_Vect, &
     Net_Rate, Net_Err, Bkg_Rate, Bkg_Err, Nlight, Nbin, LabelAI, Content,  &
       backapp, Col_Rate, Col_Backe, Good_Bin, locnt, hicnt, lobkg, hibkg, &
           Stringb, Stringc, Lengthb, Lengthc, unitc, unitb)

    
    call open_plot(Device_Type, Output_File)

    call draw_plot(Content, backapp, LabelAI, Time_Vect, Net_Rate, Net_Err, &
         Bkg_Rate, Bkg_Err, Nlight, Nbin, locnt, hicnt, lobkg, hibkg, &
           Stringb, Stringc, Lengthb, Lengthc, unitc, unitb)

    call recover_counts(Bin_Width*Binning_Factor, Net_Rate, Bkg_Rate, &
         Net_Count, Net_Total, Good_Bin, Nlight, Nbin)

    deallocate(Net_Rate, Net_Err, Bkg_Rate, Bkg_Err, Time_Vect, Good_Bin)


    if (Nbin > 2) then
       call variability_tests(Bin_Width*Binning_Factor, Net_Count, Net_Total,&
                              Nlight, Nbin, Which_Tests)

    else
       call warning('LCPLOT','Not enough bins to process variability tests')
    end if


    call pgend
    deallocate(Net_Count)

  end subroutine lcplot

  !-------------------------------------------------------------------------------------
  subroutine read_file(Input_File, Binning_Factor, Bin_Width, Time_Vect, &
   Net_Rate, Net_Err, Bkg_Rate, Bkg_Err, Nlight, Nbin, LabelAI, Content, &
    backapp, Col_Rate, Col_Backe, Good_Bin, locnt, hicnt, lobkg, hibkg, &
     Stringb, Stringc, Lengthb, Lengthc, unitc, unitb)
  !-------------------------------------------------------------------------------------
  ! Reads source time series FITS file and recovers all data needed. 
  ! Also creates time vector.

    character(len=*),intent(in) :: Input_File
    real(kind=single),intent(out) :: Bin_Width
    real(kind=single),dimension(:),pointer :: Time_Vect
    real(kind=single),dimension(:,:),pointer :: Net_Rate, Net_Err, Bkg_Rate 
    real(kind=single),dimension(:,:),pointer :: Bkg_Err
    character(len=1000),dimension(23),intent(out) :: LabelAI
    character(len=*),intent(out) :: Content
    integer,intent(out) :: Nlight , Nbin
    character(len=12) :: backapp, unitc, unitb
    integer :: Binning_Factor
    logical,dimension(:),pointer :: Good_Bin
    logical(boolean),dimension(:),pointer  :: gotnullvals

  ! Local variables are Set, Table, Column and Pointer needed by DAL to recover FITS
  ! binary table. Moreover Time_Zero contains the start exposure time in MJD

    type( DataSetT ) :: Ts_Set
    type( TableT ) :: Ts_Table
    type( ColumnT ) :: Col_Rate , Col_Error , Col_Backv , Col_Backe
    real(kind=single),dimension(:,:),pointer :: Array_Rate , Array_Error
    real(kind=single),dimension(:,:),pointer :: Array_Backv , Array_Backe
    real(kind=single) :: Exposure_Time
    real(kind=single) :: Time_Zero
    character(len=30) :: Stringb, Stringc
    integer :: i, j, Lengthb, Lengthc, bbin, cbin, ibin, num_NULL
    real(kind=single) :: locnt, hicnt, lobkg, hibkg, sumcounts, sumbkg, mean

    call read_data (Input_File , Ts_Set, Content, backapp)

! The keywords TSTART, TIMEDEL, and EXPOSURE TIME are in the RATE extension 
! not anymore in the primary header

    call read_table(Input_File, Ts_Set, Ts_Table, Time_Zero, Bin_Width, &
                                                     Exposure_Time)

    ibin = numberOfRows(Ts_Table)

    Col_Rate = Column(Ts_Table , ColumnNumber(Ts_Table,'RATE'),Read)
    Col_Error = Column(Ts_Table , ColumnNumber(Ts_Table,'ERROR'),Read)
    Col_Backv = Column(Ts_Table , ColumnNumber(Ts_Table,'BACKV'),Read)
    Col_Backe = Column(Ts_Table , ColumnNumber(Ts_Table,'BACKE'),Read)

    Array_Rate => real32VectorData(Col_Rate)
    Array_Error => real32VectorData(Col_Error)
    Array_Backv => real32VectorData(Col_Backv)
    Array_Backe => real32VectorData(Col_Backe)

    allocate( Net_Rate( size(Array_Rate(:,0)) , size(Array_Rate(0,:)) ) )
    allocate( Net_Err( size(Array_Error(:,0)) , size(Array_Error(0,:)) ) )
    allocate( Bkg_Rate( size(Array_Backv(:,0)) , size(Array_Backv(0,:)) ) )
    allocate( Bkg_Err( size(Array_Backe(:,0)) , size(Array_Backe(0,:)) ) )
 
    Net_Rate = Array_Rate 
    Net_Err = Array_Error
    Bkg_Rate = Array_Backv ; Bkg_Err = Array_Backe


    !**************************************
    !**************************************
    ! Check for NaN values and set these values to zero : parts from Vladimir
    !**************************************
    !**************************************
    do i=1,size(Array_Rate(:,0))
     num_NULL = 0
      do j=1,size(Array_Rate(0,:))
          if (( (Net_Rate(i,j) > 0.0) .EQV. (Net_Rate(i,j) <= 0.0)) &
               .or. (Net_Rate(i,j)/=Net_Rate(i,j)) ) then 
             ! the value is NaN: nullify it
             num_NULL = num_NULL + 1
             Net_Rate(i,j)=0.0
             Net_Err(i,j)=0.0
          endif
          if (( (Bkg_Rate(i,j) > 0.0) .EQV. (Bkg_Rate(i,j) <= 0.0)) &
               .or. (Bkg_Rate(i,j)/=Bkg_Rate(i,j)) ) then
             ! the value is NaN: nullify it 
             Bkg_Rate(i,j)=0.0
             Bkg_Err(i,j)=0.0
          endif
       end do
       if ((ibin - num_NULL) < 1) call warning('LCPLOT','No valid data points')
    end do

    !**************************************
    !**************************************

!Work out the maximum and minimum values to plot

    locnt = min(0.0,minval(Net_Rate-Net_Err))
    hicnt = maxval(Net_Rate+Net_Err)
    lobkg = min(0.0,minval(Bkg_Rate-Bkg_Err))
    hibkg = maxval(Bkg_Rate+Bkg_Err)

    call check_sizes (Net_Rate, Net_Err, Bkg_Rate, Bkg_Err, Nlight, Nbin)

    allocate( Time_Vect(Nbin) )
    do i=1,Nbin
       Time_Vect(i) = (i-0.5)*Bin_Width
    end do

    call delete_gap_data(Time_Vect, Net_Rate, Net_Err, Bkg_Rate, Bkg_Err, &
       Nlight, Nbin)

    call rebin(Binning_Factor, Bin_Width, Time_Vect, Net_Rate, Net_Err, &
   Bkg_Rate, Bkg_Err, Good_Bin, Nlight, Nbin, Col_Rate, Col_Backe, gotnullvals)
    call create_labels(Input_File, Nbin, Binning_Factor, Ts_Set , LabelAI)
   

!Work out the mean values and write them as characters to pass to the plot

    cbin=0; bbin=0; sumcounts=0.0; sumbkg=0.0
    ridgroupnulls: do j=1,Nlight
      thisloop: do i=1,Nbin-1
      if (gotnullvals(i) .or. Net_Rate(j,i) == 1.0e10) cycle thisloop
        cbin=cbin+1
        sumcounts=sumcounts+Net_Rate(j,i)
     end do thisloop
    end do ridgroupnulls
    ridnulls: do j=1,Nlight
     tloop: do i=1,Nbin
      if (Bkg_Rate(j,i) == 1.0e10) cycle tloop
        bbin=bbin+1
        sumbkg=sumbkg+Bkg_Rate(j,i)
     end do tloop
    end do ridnulls


    if (sumcounts > 0.0) then
      mean=abs(sumcounts/real(cbin))
!HAVING PROBS HERE FORMATTING
      call string_of_real(mean, Stringc, Lengthc)
      unitc = ' '
    else
      mean=abs(sumcounts/real(cbin))
      call string_of_real(mean, Stringc, Lengthc)
      unitc = '-'
    end if

    if (sumbkg > 0.0) then
      mean=abs(sumbkg/real(bbin))
      call string_of_real(mean, Stringb, Lengthb)
      unitb = ' '
    else
      mean=abs(sumbkg/real(bbin))
      call string_of_real(mean, Stringb, Lengthb)
      unitb = '-'
    end if

    call release(Col_Rate)
    call release(Col_Error)
    call release(Col_Backv)
    call release(Col_Backe)
    call release(Ts_Table)    
    call release(Ts_Set)

  end subroutine read_file

  !-------------------------------------------------------------------------------------
  subroutine read_data (Input_File , Ts_Set, Content, backapp)
  !-------------------------------------------------------------------------------------
  ! Reads the in parameter FITS file and gives back a handle to corresponding 
  ! dataset.  Checks some keywords existence and validity.

    character(len=*),intent(in) :: Input_File 
    type( DataSetT ),intent(out) :: Ts_Set
    character(len=*),intent(out) :: Content
    character(len=12),intent(out) :: BACKAPP

    type( TableT ) :: Table1

!    onoffvar = setExists(Input_File)
!    if (.not. onoffvar ) call error('LCPLOT','Input file does not exist : '//Input_File)
!    The previous message was misleading. It is the dal responsability (DataSet) to check
!    whether the input file is valid, has the right format, and so on. This fixes SPR-1530

    Ts_Set = dataSet(Input_File,READ)

    Table1 = table(Ts_Set,BlockNumber(Ts_Set,'RATE')) 

    if (HasAttribute(Ts_Set,"BACKAPP") ) then
         backapp = StringAttribute(Ts_Set,'BACKAPP')
    else if (HasAttribute(Table1,"BACKAPP") ) then
         backapp = StringAttribute(Table1,'BACKAPP')
    else
         backapp = 'F'
    endif

    if ( .not. HasAttribute(Ts_Set,"CONTENT") ) then
      call warning('LCPLOT','Keyword CONTENT missing in '//Input_File)
        Content = 'SOURCE TIMESERIES'
    else
        Content = change_case(StringAttribute(Ts_Set,"CONTENT"),2)
       if ( ( Content /= 'EPIC SOURCE TIMESERIES' ) .and. &
         ( Content /='OM OSW SOURCE TIMESERIES' ) ) &
           call warning('LCPLOT','CONTENT parameter for '//Input_File// &
                          ' is : '//Content)
    endif


  end subroutine read_data

  !-------------------------------------------------------------------------------------
  subroutine read_table (Input_File, Ts_Set, Ts_Table, Time_Zero, Bin_Width, &
       Exposure_Time )
  !-------------------------------------------------------------------------------------
  ! With FITS file name and handle to dataset, generates table.
  ! Checks more keywords.
  ! Also gives back Time_Zero, Bin_Width, Exposure_Time and Content.

    character(len=*),intent(in) :: Input_File
    type( DataSetT ),intent(in) :: Ts_Set
    type( TableT ),intent(out) :: Ts_Table

    real(kind=single),intent(out) :: Time_Zero, Bin_Width , Exposure_Time

    Ts_Table = table(Ts_Set,BlockNumber(Ts_Set,'RATE'))
    if ( .not. HasAttribute(Ts_Table,"HDUCLASS") ) &
      call warning('LCPLOT','Keyword HDUCLASS missing in '//Input_File)
    if ( change_case(StringAttribute(Ts_Table,"HDUCLASS"),2) /= 'OGIP' ) &
      call warning('LCPLOT','HDUCLAS is not equal to OGIP')
    if ( .not. HasAttribute(Ts_Table,"HDUCLAS1") ) &
      call warning('LCPLOT','Keyword HDUCLAS1 missing in '//Input_File)
    if ( change_case(StringAttribute(Ts_Table,"HDUCLAS1"),2) /= 'LIGHTCURVE' )&
      call warning('LCPLOT','HDUCLAS1 is not equal to LIGHTCURVE')
    if ( .not. HasAttribute(Ts_Table,"HDUCLAS2") ) &
      call warning('LCPLOT','Keyword HDUCLAS2 missing in '//Input_File)
    if ( change_case(StringAttribute(Ts_Table,"HDUCLAS2"),2) /= 'TOTAL' ) &
      call warning('LCPLOT','HDUCLAS2 is not equal to TOTAL')
    if ( .not. HasAttribute(Ts_Table,"HDUCLAS3")) &
      call warning('LCPLOT','Keyword HDUCLAS3 missing in '//Input_File)
    if ( change_case(StringAttribute(Ts_Table,"HDUCLAS3"),2) /= 'RATE' ) &
      call warning('LCPLOT','HDUCLAS3 is not equal to RATE')

    if ( .not. HasAttribute(Ts_Table,"TSTART") ) then
      call warning('LCPLOT','Keyword TSTART missing in '//Input_File)
    else
      Time_Zero = real32Attribute(Ts_Table,'TSTART')
    endif

    if ( .not. HasAttribute(Ts_Table,"TIMEDEL") ) &
      call error('LCPLOT','Keyword TIMEDEL missing in '//Input_File)
    Bin_Width = real32Attribute(Ts_Table,'TIMEDEL')
    if ( Bin_Width < epsilon(0.) ) call error('LCPLOT', &
      'Null or negative Bin_Width in '//Input_File)

    if ( .not. HasAttribute(Ts_Table,"EXPOSURE") ) then
         call warning('LCPLOT','Keyword EXPOSURE missing in '//Input_File)
    else
         Exposure_Time = real32Attribute(Ts_Table,"EXPOSURE") 
         if ( Exposure_Time < epsilon(0.) ) call warning('LCPLOT', &
          'Null or negative exposure time in '//Input_File)
    endif

  end subroutine read_table


  !-------------------------------------------------------------------------------------
  subroutine check_sizes(Net_Rate, Net_Err, Bkg_Rate, Bkg_Err, Nlight, Nbin)
  !-------------------------------------------------------------------------------------
  ! Checks if all these vectors have same dimensions : Nlight * Nbin.

    real(kind=single),dimension(:,:),pointer :: Net_Rate , Net_Err , Bkg_Rate , Bkg_Err
    integer,intent(out) :: Nlight, Nbin

    Nlight = size(Net_Rate(:,1))
    if ( size(Net_Err(:,1)) /= Nlight ) call error('LCPLOT', &
         'Different number of columns between RATE, ERROR, BACKV and BACKE')
    if ( size(Bkg_Rate(:,1)) /= Nlight ) call error('LCPLOT', &
         'Different number of columns between RATE, ERROR, BACKV and BACKE')
    if ( size(Bkg_Err(:,1)) /= Nlight ) call error('LCPLOT', &
         'Different number of columns between RATE, ERROR, BACKV and BACKE')
    if ( Nlight > 9 ) call error('LCPLOT', &
         'Number of light-curves contained in a file must not exceed 9')
    if (.not. ( size(Net_Rate(1,:)) == size(Net_Err(1,:)) &
         .and. size(Bkg_Rate(1,:)) == size(Bkg_Err(1,:)) &
         .and. size(Net_Rate(1,:)) == size(Bkg_Rate(1,:)) )) call error('LCPLOT' ,&
         'Different number of rows in columns RATE, ERROR, BACKV and BACKE')
    Nbin = size( Net_Rate(1,:) )
    if ( Nbin <= 0 ) call error('LCPLOT','No data point received')

  end subroutine check_sizes 

  !-------------------------------------------------------------------------------------
  subroutine create_labels(Input_File, Nbin, Binning_Factor, Ts_Set , LabelAI)
  !-------------------------------------------------------------------------------------
  ! Generates labels to add on plot.
 
    type( dataSetT ),intent(in) :: Ts_set
    character(len=1000),dimension(23),intent(out) :: LabelAI

    character(len=12) :: TELESCOP, INSTRUME, DETNAM, DATAMODE, OBJECT
    character(len=12) :: OBS_ID, OSW_ID
    character(len=5)  :: PSF_FRAC
    character(len=8)  :: RA
    character(len=9)  :: DEC
    character(len=12) :: EXPOSURE, SRCNO, PSF_CAPP, NPIXSOU, NPIXBACK
    character(len=30) :: DATE_OBS, DATE_END,TSTART, EXP_ID
    character(len=30) :: TIMEDEL, plotbinsizec, OBSERVER 
    character(len=40) :: Input_File 
    real(kind=double) :: TIME, plotbinsize
    integer :: Length, Binning_Factor, Nbin
 

    type( TableT ) :: Table1

    Table1 = table(Ts_Set,BlockNumber(Ts_Set,'RATE')) 

    if (HasAttribute(Ts_Set,"EXP_ID") ) then
         EXP_ID = StringAttribute(Ts_Set,'EXP_ID')
    else
         EXP_ID = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"SRCNO") ) then
         SRCNO = StringAttribute(Ts_Set,'SRCNO')
    else
         SRCNO = 'Indef'
    endif
 
    if (HasAttribute(Ts_Set,"RA_OBJ") ) then
         RA = StringAttribute(Ts_Set,'RA_OBJ')
    else if (HasAttribute(Table1,"RA_OBJ") ) then
         RA = StringAttribute(Table1,'RA_OBJ')
    else
         RA = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DEC_OBJ") ) then
         DEC = StringAttribute(Ts_Set,'DEC_OBJ')
    else if (HasAttribute(Table1,"DEC_OBJ") ) then
         DEC = StringAttribute(Table1,'DEC_OBJ')
    else
         DEC = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"PSF_FRAC") ) then
         PSF_FRAC = StringAttribute(Ts_Set,'PSF_FRAC')
    else if (HasAttribute(Table1,"PSF_FRAC") ) then
         PSF_FRAC = StringAttribute(Table1,'PSF_FRAC')
    else
         PSF_FRAC = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"PSF_CAPP") ) then
         PSF_CAPP = StringAttribute(Ts_Set,'PSF_CAPP')
    else if (HasAttribute(Table1,"PSF_CAPP") ) then
         PSF_CAPP = StringAttribute(Table1,'PSF_CAPP')
    else
         PSF_CAPP = 'Indef'
    endif

    if (trim(PSF_CAPP) == 'T') then
      PSF_CAPP = 'YES'
    else if (trim(PSF_CAPP) == 'F') then
      PSF_CAPP = 'NO'
    else
      PSF_CAPP = 'Indef'
    end if

    if (HasAttribute(Ts_Set,"NPIXSOU") ) then
         NPIXSOU = StringAttribute(Ts_Set,'NPIXSOU')
    else if (HasAttribute(Table1,"NPIXSOU") ) then
         NPIXSOU = StringAttribute(Table1,'NPIXSOU')
    else
         NPIXSOU = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"NPIXBACK") ) then
         NPIXBACK = StringAttribute(Ts_Set,'NPIXBACK')
    else if (HasAttribute(Table1,"NPIXBACK") ) then
         NPIXBACK = StringAttribute(Table1,'NPIXBACK')
    else
         NPIXBACK = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"TIMEDEL") ) then
         TIME = real64Attribute(Ts_Set,'TIMEDEL')
         plotbinsize=TIME*real(Binning_Factor)
!         Binning_Factor = int(real(Nbin-1)/500.0)
         plotbinsize=plotbinsize*Binning_Factor
         call string_of_real2(plotbinsize, plotbinsizec, Length)
         call string_of_real2(TIME, TIMEDEL, Length)
    else if (HasAttribute(Table1,"TIMEDEL") ) then
         TIME = real64Attribute(Table1,'TIMEDEL')
         plotbinsize=TIME*real(Binning_Factor)
!         Binning_Factor = int(real(Nbin-1)/500.0)
         plotbinsize=plotbinsize*Binning_Factor
         call string_of_real2(plotbinsize, plotbinsizec, Length)
         call string_of_real2(TIME, TIMEDEL, Length)
    else
         TIMEDEL = 'Indef'
    endif
 
    if (HasAttribute(Ts_Set,"TSTART") ) then
         TIME = real64Attribute(Ts_Set,'TSTART')
         call string_of_real2(TIME, TSTART, Length)
    else if (HasAttribute(Table1,"TSTART") ) then
         TIME = real64Attribute(Table1,'TSTART')
         call string_of_real2(TIME, TSTART, Length)
    else
         TSTART = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"EXPOSURE") ) then
         EXPOSURE = StringAttribute(Ts_Set,'EXPOSURE')
    else if (HasAttribute(Table1,"EXPOSURE") ) then
         EXPOSURE = StringAttribute(Table1,'EXPOSURE')
    else
         EXPOSURE = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DATE-END") ) then
         DATE_END = StringAttribute(Ts_Set,'DATE-END')
    else if (HasAttribute(Table1,"DATE-END") ) then
         DATE_END = StringAttribute(Table1,'DATE-END')
    else
         DATE_END = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DATE-OBS") ) then
         DATE_OBS = StringAttribute(Ts_Set,'DATE-OBS')
    else if (HasAttribute(Table1,"DATE-OBS") ) then
         DATE_OBS = StringAttribute(Table1,'DATE-OBS')
    else
         DATE_OBS = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OBS_ID") ) then
         OBS_ID = StringAttribute(Ts_Set,'OBS_ID')
    else if (HasAttribute(Table1,"OBS_ID") ) then
         OBS_ID = StringAttribute(Table1,'OBS_ID')
    else
         OBS_ID = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OSW_ID") ) then
         OSW_ID = StringAttribute(Ts_Set,'OSW_ID')
    else if (HasAttribute(Table1,"OSW_ID") ) then
         OSW_ID = StringAttribute(Table1,'OSW_ID')
    else
         OSW_ID = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OBJECT") ) then
         OBJECT = StringAttribute(Ts_Set,'OBJECT')
    else if (HasAttribute(Table1,"OBJECT") ) then
         OBJECT = StringAttribute(Table1,'OBJECT')
    else
         OBJECT = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OBSERVER") ) then
         OBSERVER = StringAttribute(Ts_Set,'OBSERVER')
    else if (HasAttribute(Table1,"OBSERVER") ) then
         OBSERVER = StringAttribute(Table1,'OBSERVER')
    else
         OBSERVER = 'Indef'
    endif

   if (HasAttribute(Ts_Set,"DATAMODE") ) then
         DATAMODE = StringAttribute(Ts_Set,'DATAMODE')
    else if (HasAttribute(Table1,"DATAMODE") ) then
         DATAMODE = StringAttribute(Table1,'DATAMODE')
    else
         DATAMODE = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DETNAM") ) then
         DETNAM = StringAttribute(Ts_Set,'DETNAM')
    else if (HasAttribute(Table1,"DETNAM") ) then
         DETNAM = StringAttribute(Table1,'DETNAM')
    else
         DETNAM = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"INSTRUME") ) then
         INSTRUME = StringAttribute(Ts_Set,'INSTRUME')
    else if (HasAttribute(Table1,"INSTRUME") ) then
         INSTRUME = StringAttribute(Table1,'INSTRUME')
    else
         INSTRUME = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"TELESCOP") ) then
         TELESCOP = StringAttribute(Ts_Set,'TELESCOP')
    else if (HasAttribute(Table1,"TELESCOP") ) then
         TELESCOP = StringAttribute(Table1,'TELESCOP')
    else
         TELESCOP = 'Indef'
    endif

    LabelAI(1) = trim(TELESCOP)
    LabelAI(4) = trim(INSTRUME)
    LabelAI(7) = trim(DETNAM)
    LabelAI(10) = trim(DATAMODE)
    LabelAI(23) = 'FILE: '//trim(Input_File)
    LabelAI(11) = 'PI: '//trim(OBSERVER)
    LabelAI(2) = 'OBJECT: '//trim(OBJECT)
    LabelAI(5) = 'OBS ID: '//trim(OBS_ID)
    LabelAI(8) = 'EXP ID: '//trim(EXP_ID)
    LabelAI(3) = 'OSW ID: '//trim(OSW_ID)
    LabelAI(14) = 'SRC NO: '//trim(SRCNO)
    LabelAI(19) = 'SRC RA: '//trim(RA)
    LabelAI(22) = 'SRC DEC: '//trim(DEC)
    LabelAI(9) = 'SRC EXTR. AREA: '//trim(NPIXSOU)//' pix'
    LabelAI(12) = 'BKGRND EXTR. AREA: '//trim(NPIXBACK)//' pix'
    LabelAI(13) = 'PSF FRAC. INC: '//trim(PSF_FRAC)
    LabelAI(16) = 'VIGNET. CORR: '//trim(PSF_CAPP)
    LabelAI(15) = 'START: '//trim(DATE_OBS)
    LabelAI(18) = 'STOP: '//trim(DATE_END)
    LabelAI(21) = 'EXP: '//trim(EXPOSURE)//' s'
    LabelAI(6) = 'TIME START: '//trim(TSTART)//' MJD'
    LabelAI(17) = 'ORIG. BIN: '//trim(TIMEDEL)//' s'
    LabelAI(20) = 'PLOTTED BIN SIZE: ' //trim(plotbinsizec)// ' s'

  end subroutine create_labels

  
  !-------------------------------------------------------------------------------------
  subroutine delete_gap_data(Time_Vect, Net_Rate, Net_Err, Bkg_Rate, Bkg_Err, &
       Nlight, Nbin)
  !-------------------------------------------------------------------------------------
  ! Deletes gap (when net and background rates are both 0.0)

    real(kind=single),dimension(:),pointer :: Time_Vect
    real(kind=single),dimension(:,:),pointer :: Net_Rate, Net_Err, Bkg_Rate 
    real(kind=single),dimension(:,:),pointer :: Bkg_Err
    integer,intent(inout) :: Nbin
    integer,intent(in) :: Nlight

    real(kind=single),dimension(Nbin) :: Time_Vect_Temp
    real(kind=single),dimension(Nlight,Nbin) :: Net_Rate_Temp , Net_Err_Temp
    real(kind=single),dimension(Nlight,Nbin) :: Bkg_Rate_Temp , Bkg_Err_Temp
    real(kind=single) :: Eps
    integer :: Nbin_Temp , Light , i
    logical :: Good_Value

    Eps = epsilon(0.)
    Nbin_Temp = 0
    do i=1,Nbin
       Good_Value = .true.
       do Light=1,Nlight
!          Good_Value = Good_Value .and. ( ( Net_Rate(Light,i) > Eps ) .or. &
!
!               ( Bkg_Rate(Light,i) > Eps ) )
         Good_Value = Good_Value 
       end do
       if (Good_Value) then
          Nbin_Temp = Nbin_Temp + 1
          Time_Vect_Temp(Nbin_Temp) = Time_Vect(i)
          Net_Rate_Temp(:,Nbin_Temp) = Net_Rate(:,i)
          Net_Err_Temp(:,Nbin_Temp) = Net_Err(:,i)
          Bkg_Rate_Temp(:,Nbin_Temp) = Bkg_Rate(:,i)
          Bkg_Err_Temp(:,Nbin_Temp) = Bkg_Err(:,i)
       end if
    end do
    deallocate(Time_Vect)
    deallocate(Net_Rate , Net_Err , Bkg_Rate , Bkg_Err)
    Nbin = Nbin_Temp
    allocate( Time_Vect(Nbin) )
    Time_Vect = Time_Vect_Temp
    allocate( Net_Rate(Nlight,Nbin) , Net_Err(Nlight,Nbin) )
    allocate( Bkg_Rate(Nlight,Nbin) , Bkg_Err(Nlight,Nbin) )
    Net_Rate = Net_Rate_Temp ; Net_Err = Net_Err_Temp
    Bkg_Rate = Bkg_Rate_Temp ; Bkg_Err = Bkg_Err_Temp
    if ( minval(Net_Rate) < 0.0 .or. minval(Bkg_Rate)  < 0.0 ) &
         call warning('LCPLOT','Count rate is not positive')
    if ( minval(Net_Err) < 0.0 .or. minval(Bkg_Err) < 0.0 ) &
         call warning('LCPLOT','Count rate errors must be positive')

    if (size(Time_Vect) < 1) then
         call error('LCPLOT','No good data points')
    end if

  end subroutine delete_gap_data

  !-------------------------------------------------------------------------------------
  subroutine rebin(Binning_Factor, Bin_Width, Time_Vect, Net_Rate, Net_Err, &
  Bkg_Rate, Bkg_Err, Good_Bin, Nlight, Nbin, Col_Rate, Col_Backe, gotnullvals)
  !-------------------------------------------------------------------------------------
  ! Rebins Net_Rate, Net_Err, Bkg_Rate, Bkg_Err by at least a factor five 
  ! to ensure good signal-to-noise and to limit the number of plotted points 
  ! to 500.  Also modifies Time_Vect.

    integer,intent(inout) :: Binning_Factor
    real(kind=single),dimension(:,:),pointer :: Net_Rate, Net_Err, Bkg_Rate, Bkg_Err
    integer,intent(in) :: Nlight
    integer,intent(inout) :: Nbin
    real(kind=single),intent(in) :: Bin_Width
    real(kind=single),dimension(:),pointer :: Time_Vect
    logical,dimension(:),pointer :: Good_Bin
    type( ColumnT ) :: Col_Rate, Col_Backe

    real(kind=single),dimension(:,:),pointer :: Net_Rate_Reb, Net_Err_Reb
    real(kind=single),dimension(:,:),pointer :: Bkg_Rate_Reb, Bkg_Err_Reb
    real(kind=single),dimension(:),pointer :: Time_Reb
    real(kind=single),dimension(Nbin) :: Time_Vect_Temp
    real(kind=single),dimension(Nlight,Nbin) :: Net_Rate_Temp, Net_Err_Temp
    real(kind=single),dimension(Nlight,Nbin) :: Bkg_Rate_Temp, Bkg_Err_Temp
    logical(boolean),dimension(:),pointer  :: gotnullvals
    integer :: Nbin_Reb ,  i , j, Nbin_Temp, Light, k
    logical :: No_Gap, Good_Value
    real :: Eps, averr

    ! Sets Binning_Factor to correct value
    if (Binning_Factor < 1) then
       Binning_Factor = 1
       call warning('LCPLOT','Binning_Factor must be positive, set to default: 1')
    end if
    if ( int(Nbin/Binning_Factor)>500 ) then
       Binning_Factor = ((int(Nbin/Binning_Factor)-1)/512 + 1)*Binning_Factor
       write(errstr,*) 'Data point to plot must not exceed 500. '//&
                  'Binning Factor is increased from input value to : ',Binning_Factor
       call warning('LCPLOT')
    end if

    ! Rebinning

    allocate(Time_Reb(Nbin))
    allocate(Good_Bin(Nbin))
    allocate(gotnullvals(Nbin))
    allocate(Net_Rate_Reb(Nlight,Nbin))
    allocate(Net_Err_Reb(Nlight,Nbin))
    allocate(Bkg_Rate_Reb(Nlight,Nbin))      
    allocate(Bkg_Err_Reb(Nlight,Nbin))      

    Eps = Bin_Width/10.
    Nbin_Reb = 0
    i = 0
    do while (i < Nbin)
!       Nbin_Reb                 =  Nbin_Reb + 1
!       Time_Reb(Nbin_Reb)       =  Time_Vect(i)
!       Net_Rate_Reb(:,Nbin_Reb) =  Net_Rate(:,i)
!       Net_Err_Reb(:,Nbin_Reb)  =  Net_Err(:,i)**2.0
!       Bkg_Rate_Reb(:,Nbin_Reb) =  Bkg_Rate(:,i)
!       Bkg_Err_Reb(:,Nbin_Reb)  =  Bkg_Err(:,i)**2.0
!       i = i + 1 ; j = 1
!       No_Gap=(abs(Time_Vect(min(i,Nbin))-Time_Vect(i-1)-Bin_Width) <= Eps)
!       do while ( ( (i <= Nbin) .and. (j < Binning_Factor) ))
!          Net_Rate_Reb(:,Nbin_Reb) = Net_Rate_Reb(:,Nbin_Reb) + Net_Rate(:,i)
!          Net_Err_Reb(:,Nbin_Reb) = Net_Err_Reb(:,Nbin_Reb) + Net_Err(:,i)**2.0
!          Bkg_Rate_Reb(:,Nbin_Reb) = Bkg_Rate_Reb(:,Nbin_Reb) + Bkg_Rate(:,i)
!          Bkg_Err_Reb(:,Nbin_Reb) = Bkg_Err_Reb(:,Nbin_Reb) + Bkg_Err(:,i)**2.0
!          j = j + 1 ; i = i + 1
!          No_Gap=(abs(Time_Vect(min(i,Nbin))-Time_Vect(i-1)-Bin_Width) <= Eps)
!       end do
!       Net_Rate_Reb(:,Nbin_Reb)  =  Net_Rate_Reb(:,Nbin_Reb) / j
!       Net_Err_Reb(:,Nbin_Reb)   =  sqrt(Net_Err_Reb(:,Nbin_Reb)) / j
!       Bkg_Rate_Reb(:,Nbin_Reb)  =  Bkg_Rate_Reb(:,Nbin_Reb) / j
!       Bkg_Err_Reb(:,Nbin_Reb)   =  sqrt(Bkg_Err_Reb(:,Nbin_Reb)) / j
!       Good_Bin(Nbin_Reb)        =  ( j == Binning_Factor )
!    end do
!    deallocate(Time_Vect)
!    deallocate(Net_Rate , Net_Err , Bkg_Rate , Bkg_Err)
!    Nbin = Nbin_Reb
!    allocate(Time_Vect(Nbin))
!    allocate(Net_Rate(Nlight,Nbin) , Net_Err(Nlight,Nbin) )
!    allocate(Bkg_Rate(Nlight,Nbin) , Bkg_Err(Nlight,Nbin) )
!    Time_Vect  =  Time_Reb(1:Nbin)
!    Net_Rate   =  Net_Rate_Reb(:,1:Nbin)
!    Net_Err    =  Net_Err_Reb(:,1:Nbin)
!    Bkg_Rate   =  Bkg_Rate_Reb(:,1:Nbin)
!    Bkg_Err    =  Bkg_Err_Reb(:,1:Nbin)
!    deallocate(Net_Rate_Reb , Net_Err_Reb , Bkg_Rate_Reb , Bkg_Err_Reb)
       i = i + 1  
       Nbin_Reb                 =  Nbin_Reb + 1
       Time_Reb(Nbin_Reb)       =  Time_Vect(i)
       Net_Rate_Reb(:,Nbin_Reb) =  Net_Rate(:,i)
       if ( isNull ( Col_Rate , i-1 )) then
!          num_NULL = num_NULL + 1
          Net_Err_Reb(:,Nbin_Reb)  =  0.0
          gotnullvals(Nbin_Reb)    = .true.
       else
         Net_Err_Reb(:,Nbin_Reb)  =  Net_Err(:,i)**2.0
         gotnullvals(Nbin_Reb)    = .false.
       end if
       Bkg_Rate_Reb(:,Nbin_Reb) =  Bkg_Rate(:,i)
       if ( isNull ( Col_Backe , i-1 ) ) then
         Bkg_Err_Reb(:,Nbin_Reb)  =  0.0
       else 
         Bkg_Err_Reb(:,Nbin_Reb)  =  Bkg_Err(:,i)**2.0
       end if
       j = 1; k = 1
!       No_Gap=(abs(Time_Vect(min(i,Nbin))-Time_Vect(i-1)-Bin_Width) <= Eps)
       grouploop: do while ( ( (i < Nbin) .and. (k < Binning_Factor) ))
          i=i+1; k=k+1
          Time_Reb(Nbin_Reb) = Time_Reb(Nbin_Reb) + Time_Vect(i)
        if (isNull( Col_Rate , i-1 ) .and. .not. isNull( Col_Rate , i-2 )) then
          Net_Rate_Reb(:,Nbin_Reb) = Net_Rate_Reb(:,Nbin_Reb) + 0.0 
          Net_Err_Reb(:,Nbin_Reb) =  Net_Err_Reb(:,Nbin_Reb) + 0.0 
          Bkg_Rate_Reb(:,Nbin_Reb) =  Bkg_Rate_Reb(:,Nbin_Reb) + 0.0 
          Bkg_Err_Reb(:,Nbin_Reb) = Bkg_Err_Reb(:,Nbin_Reb) + 0.0 
          gotnullvals(Nbin_Reb)   = .true.
        else if (isNull( Col_Rate , i-2 ) .and. .not. isNull( Col_Rate , i-1 )) then
          Net_Rate_Reb(:,Nbin_Reb) = 0.0 + Net_Rate(:,i)
          Net_Err_Reb(:,Nbin_Reb) = 0.0 + Net_Err(:,i)**2.0
          Bkg_Rate_Reb(:,Nbin_Reb) = 0.0 + Bkg_Rate(:,i)
          Bkg_Err_Reb(:,Nbin_Reb) = 0.0 + Bkg_Err(:,i)**2.0
          gotnullvals(Nbin_Reb)   = .true.
        else if (isNull( Col_Rate , i-1 ) .and. isNull( Col_Rate , i-2 )) then
          Net_Rate_Reb(:,Nbin_Reb) =   1.0e10
          Net_Err_Reb(:,Nbin_Reb) =  0.0
          Bkg_Rate_Reb(:,Nbin_Reb) =  1.0e10
          Bkg_Err_Reb(:,Nbin_Reb) =   0.0
!          Net_Rate_Reb(:,Nbin_Reb) =   Net_Rate(:,i)
!          Net_Err_Reb(:,Nbin_Reb) =  Net_Err(:,i)
!          Bkg_Rate_Reb(:,Nbin_Reb) =  Bkg_Rate(:,i)
!          Bkg_Err_Reb(:,Nbin_Reb) =   Bkg_Err(:,i)
          gotnullvals(Nbin_Reb)   = .true.
        else 
          Net_Rate_Reb(:,Nbin_Reb) = Net_Rate_Reb(:,Nbin_Reb) + Net_Rate(:,i)
          Net_Err_Reb(:,Nbin_Reb) = Net_Err_Reb(:,Nbin_Reb) + Net_Err(:,i)**2.0
          Bkg_Rate_Reb(:,Nbin_Reb) = Bkg_Rate_Reb(:,Nbin_Reb) + Bkg_Rate(:,i)
          Bkg_Err_Reb(:,Nbin_Reb) = Bkg_Err_Reb(:,Nbin_Reb) + Bkg_Err(:,i)**2.0
          gotnullvals(Nbin_Reb)   = .false.
          j = j + 1 
        endif
          No_Gap=(abs(Time_Vect(min(i,Nbin))-Time_Vect(i-1)-Bin_Width) <= Eps)
       end do grouploop
       Time_Reb(Nbin_Reb) = Time_Reb(Nbin_Reb)/real(k)
       if (.not. gotnullvals(Nbin_Reb)) then
         Net_Rate_Reb(:,Nbin_Reb)  =  Net_Rate_Reb(:,Nbin_Reb) / real(j)
         Net_Err_Reb(:,Nbin_Reb)   =  sqrt(Net_Err_Reb(:,Nbin_Reb)) / real(j)
         Bkg_Rate_Reb(:,Nbin_Reb)  =  Bkg_Rate_Reb(:,Nbin_Reb) / real(j)
         Bkg_Err_Reb(:,Nbin_Reb)   =  sqrt(Bkg_Err_Reb(:,Nbin_Reb)) / real(j)
         Good_Bin(Nbin_Reb)        =  ( j == Binning_Factor )
!       else
!         Net_Rate_Reb(:,Nbin_Reb)  =  Net_Rate_Reb(:,Nbin_Reb)
!         Net_Err_Reb(:,Nbin_Reb)   =  Net_Err_Reb(:,Nbin_Reb)
!         Bkg_Rate_Reb(:,Nbin_Reb)  =  Bkg_Rate_Reb(:,Nbin_Reb) 
!         Bkg_Err_Reb(:,Nbin_Reb)   =  Bkg_Err_Reb(:,Nbin_Reb)
       end if 
    end do 
    deallocate(Time_Vect)
    deallocate(Net_Rate , Net_Err , Bkg_Rate , Bkg_Err)
    Nbin = Nbin_Reb
    allocate(Time_Vect(Nbin))
    allocate(Net_Rate(Nlight,Nbin), Net_Err(Nlight,Nbin))
    allocate(Bkg_Rate(Nlight,Nbin), Bkg_Err(Nlight,Nbin))
    Time_Vect  =  Time_Reb(1:Nbin)
    Net_Rate   =  Net_Rate_Reb(:,1:Nbin)
    Net_Err    =  Net_Err_Reb(:,1:Nbin)
    Bkg_Rate   =  Bkg_Rate_Reb(:,1:Nbin)
    Bkg_Err    =  Bkg_Err_Reb(:,1:Nbin)
    deallocate(Net_Rate_Reb, Net_Err_Reb, Bkg_Rate_Reb, Bkg_Err_Reb, Time_Reb)

!This loop removes bins with a low exposure and therefore large errors, from the plot
!as requested by JO, April 2002

!    cullloop: do
!      Nbin_Temp = 0
!      averr=sum(Net_Err)/Nbin
!      do i=1,Nbin
!         Good_Value = .true.
!         do Light=1,Nlight
!            Good_Value = Good_Value .and. ( ( Net_Err(Light,i) < 5.0*averr ) )
!         end do
!         if (Good_Value) then
!            Nbin_Temp = Nbin_Temp + 1
!            Time_Vect_Temp(Nbin_Temp) = Time_Vect(i)
!            Net_Rate_Temp(:,Nbin_Temp) = Net_Rate(:,i)
!            Net_Err_Temp(:,Nbin_Temp) = Net_Err(:,i)
!            Bkg_Rate_Temp(:,Nbin_Temp) = Bkg_Rate(:,i)
!            Bkg_Err_Temp(:,Nbin_Temp) = Bkg_Err(:,i)
!         end if
!      end do

!      if (Nbin == Nbin_Temp) exit cullloop

!      deallocate(Net_Rate , Net_Err , Bkg_Rate , Bkg_Err, Time_Vect)
!      Nbin=Nbin_Temp
!      allocate(Net_Rate(Nlight,Nbin) , Net_Err(Nlight,Nbin), Time_Vect(Nbin) )
!      allocate(Bkg_Rate(Nlight,Nbin) , Bkg_Err(Nlight,Nbin) )
!      Time_Vect = Time_Vect_Temp
!      Net_Rate = Net_Rate_Temp
!      Net_Err = Net_Err_Temp
!      Bkg_Rate = Bkg_Rate_Temp
!      Bkg_Err = Bkg_Err_Temp
!
!    end do cullloop

  end subroutine rebin

!-------------------------------------------------------------------------------------
  subroutine open_plot(Device_Type, Output_File) 
  !-------------------------------------------------------------------------------------
  ! Check Device_Type and Output_File. Open plot if required.

    character(len=*),intent(inout) :: Device_Type , Output_File
    integer :: pgbeg
    character(len=2048) :: Device_Selected
    logical :: File_Exists

    if (Output_File == '')  Output_File='default.ps'
    if (Device_Type(1:3) /= '/XW') then
      inquire(file=Output_File,exist=File_Exists)   
      if (File_Exists .and. .not. clobber()) call error('LCPLOT', &
      'Already existing file '//trim(Output_File)//' cannot be overwritten (SAS_CLOBBER=0)')
      if (File_Exists .and. clobber()) call warning('LCPLOT', &
      'Already existing file '//trim(Output_File)//' will be overwritten')
      Device_Selected=trim(Output_File)//trim(Device_Type)
      if ( pgbeg(0,Device_Selected,1,1)/=1 ) then
        call warning('LCPLOT', &
        'Invalid device and/or output file: '//trim(Device_Type)//' , '//trim(Output_File))
        Output_File='default.ps'
        Device_Type= '/PS'
        Device_Selected=trim(Output_File)//trim(Device_Type)
        if ( pgbeg(0,Device_Selected,1,1)/=1 ) then
          call error('LCPLOT', &
          'Invalid DEFAULT device and/or output file: '//trim(Device_Type)//' , '&
                  //trim(Output_File))
        else
          call warning('LCPLOT', &
          'New device and output file: '//trim(Device_Type)//' , '//trim(Output_File))
            endif
         endif
    else
        Device_Selected=trim(Device_Type)
        if ( pgbeg(0,Device_Selected,1,1)/=1 ) call error('LCPLOT', &
        'Invalid device : '//trim(Device_Type))
   endif
   call pgpap(0.0 , 1.618 )
    call pgask(.true.)
    call pgsch(0.9)

  end subroutine open_plot

  !-------------------------------------------------------------------------------------
  subroutine draw_plot(Content, backapp, LabelAI, Time_Vect, Net_Rate, Net_Err,&
       Bkg_Rate, Bkg_Err, Nlight, Nbin, locnt, hicnt, lobkg, hibkg, &
           Stringb, Stringc, Lengthb, Lengthc, unitc, unitb)
  !-------------------------------------------------------------------------------------
  ! Draw labelled plot in which each page contains the net and background 
  ! source time series representation of a specific light-curve.

    character(len=*),intent(in) :: Content
    character(len=1000),dimension(23),intent(in) :: LabelAI
    real(kind=single),dimension(:),pointer :: Time_Vect
    real(kind=single),dimension(:,:),pointer :: Net_Rate, Net_Err, Bkg_Rate, Bkg_Err
    integer,intent(in) :: Nlight , Nbin
    integer :: whichfile, pass
    real :: botval, topval
    character(len=12) :: backapp, unitc, unitb
    real(kind=single) :: locnt, hicnt, lobkg, hibkg, mean
    character(len=30) :: Stringb, Stringc
    integer :: Lengthb, Lengthc, bbin, cbin


  ! Local variables Window1&2 are used to define the viewports.

    real(kind=single),dimension(4) :: Window1 , Window2
    integer :: Light , i

    info = getTaskInfo()
!    call readParameters

    Window1 = (/0.1,0.95,0.50,0.73/)
    Window2 = (/0.1,0.95,0.1,0.33/)

    do Light=1,Nlight
       call pgpage
       call pgsvp(0.0,1.0,0.0,1.0)
       call pgsci(11)
    call pgmtxt("b", -1.0, 0.0, 0.0, trim(nameOf(info))//' v'//trim(versionOf(info)))
       pass=0
       do i=1,23,3
          pass=pass+1 
          call pgmtxt('T' , -(1. + (pass-1)*1.2) , 0.03 , 0.0 , LabelAI(i))
       end do
       pass=0
       do i=2,23,3
         pass=pass+1 
         call pgmtxt('T' , -(1. + (pass-1)*1.2) , 0.28 , 0.0 , LabelAI(i))
       end do
       pass=0
       do i=3,23,3
         pass=pass+1 
          call pgmtxt('T' , -(1. + (pass-1)*1.2) , 0.6 , 0.0 , LabelAI(i))
        end do
       call pgsci(11)
       call pgsch(0.9)
       call pgmtxt('T', -15.*0.5/0.6, 0.5, 0.5, 'RATES '//achar(iachar("0")+Light))
       call pgsch(0.9)
       whichfile=1

       call make_one_plot('Net time series', backapp, Window1, Time_Vect, &
        Net_Rate(Light,:), Net_Err(Light,:), Nbin, whichfile, botval, topval, &
              locnt, hicnt, lobkg, hibkg, &
           Stringb, Stringc, Lengthb, Lengthc, unitc, unitb)

       call pgsch(0.9)
       whichfile=2
       call make_one_plot('Background time series', backapp, Window2, Time_Vect, &
        Bkg_Rate(Light,:), Bkg_Err(Light,:), Nbin, whichfile, botval, topval, &
           locnt, hicnt, lobkg, hibkg, &
           Stringb, Stringc, Lengthb, Lengthc, unitc, unitb)
    end do

  end subroutine draw_plot

  !-------------------------------------------------------------------------------------
  subroutine make_one_plot(Title, backapp, Window, Time_Vect, Rate, Err , &
       Nbin, whichfile, botval, topval, locnt, hicnt, lobkg, hibkg, &
           Stringb, Stringc, Lengthb, Lengthc, unitc, unitb)
  !-------------------------------------------------------------------------------------
  ! Draw one specific plot. X-axis corresponds to time and Y-axis to count rates.
  ! Plot title and mean rate are labelled.

    character(len=*) :: Title
    real(kind=single),dimension(4),intent(in) :: Window
    real(kind=single),dimension(:),pointer :: Time_Vect
    real(kind=single),dimension(:),intent(in) :: Rate , Err
    real :: botval, topval
    integer,intent(in) :: Nbin, whichfile
    character(len=12) :: backapp, unitc, unitb
    character(len=20) :: String
    integer :: Length
    real(kind=single) :: locnt, hicnt, lobkg, hibkg, mean
    character(len=30) :: Stringb, Stringc
    integer :: Lengthb, Lengthc, bbin, cbin


    call string_of_real(sum(Rate)/real(Nbin) , String , Length )
    call pgsvp(Window(1),Window(2),Window(3),Window(4))
    if (whichfile == 1) then
      call pgswin(0. , Time_Vect(Nbin) + Time_Vect(1) , &
         locnt, hicnt )
      botval=min(0.,minval(Rate-Err))
      topval=maxval(Rate+Err) 
    else if  (whichfile == 2 .and. .not. booleanParameter("bkgdyscale")) then
      call pgswin(0. , Time_Vect(Nbin) + Time_Vect(1) , &
         lobkg, hibkg )
    else if (whichfile == 2 .and. booleanParameter("bkgdyscale")) then
      call pgswin(0. , Time_Vect(Nbin) + Time_Vect(1) , &
         locnt, hicnt )
    else
      call pgswin(0. , Time_Vect(Nbin) + Time_Vect(1) , &
         locnt, hicnt )
    end if      
    call pgsci(1)
    call pgsch(0.9)
    call pgbox('BCNST' , 0.0 , 5 ,'BCNST' , 0.0 , 5)
    if (trim(backapp) == 'T' .and. Title /= 'Background time series') then
      call pglab('Time after TSTART (sec)', 'Counts/sec', & 
        Title//', BACKGROUND SUBTRACTED - Mean Rate='// trim(unitc)//Stringc(1:Lengthc))
    else if (Title == 'Background time series') then
      call pglab('Time after TSTART (sec)', 'Counts/sec', & 
         Title//' - Mean Rate='// trim(unitb)//Stringb(1:Lengthb))
    else if (trim(backapp) == 'F' .and. whichfile  < 2) then 
      call warning('ELCPLOT', 'Non background subtracted lightcurve supplied')
      call pglab('Time after TSTART (sec)', 'Counts/sec', & 
        Title//', NON BACKGROUND SUBTRACTED - mean Rate='// trim(unitc)//Stringc(1:Lengthc))
    end if
!    if (trim(backapp) == 'T' .and. Title /= 'BACKGROUND TIME SERIES') then
!      call pglab('Time after TSTART (sec)', 'Counts/sec', & 
!        Title//', BACKGROUND SUBTRACTED - Mean Rate='// trim(unit)//Stringc(1:Lengthc))
!    else 
!      call pglab('Time after TSTART (sec)', 'Counts/sec', & 
!         Title//' - Mean Rate='//  trim(unit)//Stringb(1:Lengthb))
!    end if
    call pgsci(2)

    call pgerry(Nbin , Time_Vect , Rate+Err , Rate-Err, 0.0)
    call pgsci(1)
    call pgslw(10)

    call pgpt(Nbin , Time_Vect , Rate , -1)
    call pgslw(1)

  end subroutine make_one_plot

  !----------------------------------------------------------------------------------
  subroutine recover_counts(Time_Length, Net_Rate, Bkg_Rate, Net_Count, &
       Net_Total, Good_Bin, Nlight, Nbin)
  !----------------------------------------------------------------------------------
  ! Recovers total source counts (net*time)

    real(kind=single),intent(in) :: Time_Length
    real(kind=single),dimension(:,:),pointer :: Net_Rate , Net_Count
    real(kind=single),dimension(:,:),pointer :: Bkg_Rate , Net_Total
    real(kind=single),dimension(:,:),pointer :: Net_Count_Temp, Net_Total_Temp
    logical,dimension(:),pointer :: Good_Bin
    integer,intent(in) :: Nlight
    integer,intent(inout) :: Nbin
    integer :: i, n

    allocate(Net_Count_Temp(Nlight,Nbin))
    allocate(Net_Total_Temp(Nlight,Nbin))
    n=0
    do i=1,Nbin
       if ( Good_Bin(i) ) n = n + 1
    end do
    do i=1,Nbin
       if ( Good_Bin(i) ) then
          Net_Count_Temp(:,i) = Net_Rate(:,i) * Time_Length
          Net_Total_Temp(:,i) = (Net_Rate(:,i)+Bkg_Rate(:,i)) * Time_Length
       endif
    end do
    Nbin = n
    allocate(Net_Count(Nlight,Nbin))
    allocate(Net_Total(Nlight,Nbin))
    Net_Count = Net_Count_Temp(:,1:Nbin)
    Net_Total = Net_Total_Temp(:,1:Nbin)

    deallocate(Net_Count_Temp)
    deallocate(Net_Total_Temp)

  end subroutine recover_counts

  !-------------------------------------------------------------------------------------
  subroutine variability_tests(Bin_size, Net_Count, Net_total, Nlight, Nbin, Which_Tests)
  !-------------------------------------------------------------------------------------
  ! Performs many algorithms to determine how varies the total source. 
  ! Results are provided on the same output device than plots.

    real(kind=single),dimension(:,:),pointer :: Net_Count, Net_total
    integer,intent(in) :: Nlight , Nbin
    real,intent(in) :: Bin_size

    type(data_tests),dimension(Nlight) :: Light_Curve
    type(data_tests),dimension(Nlight) :: Light_Curve_total

    integer :: Light

    character(len=20) :: String, String_t, String_b
    character(len=12) :: Which_Tests
    integer :: Length , Length_t , Length_b , i
    real :: Character_Size , Height

    if (Nlight>1) then
       Height = 1.0 / (Nlight + 0.5 )
    else
       Height = 1.0 / 2.5
    end if
    Character_Size = Height
    call pgpage
    call pgsvp(0.0 , 1.0 , 1.0-0.5*Height , 1.0)
    call pgsch(7.5*Character_Size) ; call pgsci(3)
    call pgmtxt('T' , -1. , 0.5 , 0.5 , 'VARIABILITY TESTS')
    call pgsch(4.*Character_Size) ; call pgsci(5)
    call string_of_real(real(Nbin) , String , Length)
    call string_of_real(Bin_size , String_b , Length_b)
    call pgmtxt('T' , -4. , 0.5 , 0.5 , 'NUMBER OF BINS : '&
         //String(1:Length)//' BIN SIZE : '//String_b(1:Length_b) &
         //' sec. ')
    call pgsci(2)
    do Light=1,Nlight
       Light_Curve(Light)%Mean_Count = sum(Net_Count(Light,:)) / real(Nbin)
       Light_Curve_total(Light)%Mean_Count = sum(Net_total(Light,:)) / real(Nbin)
       Light_Curve(Light)%Variance = 0.
       Light_Curve_total(Light)%Variance = 0.
       do i=1,Nbin
          Light_Curve(Light)%Variance = Light_Curve(Light)%Variance + &
               (Net_Count(Light,i) - Light_Curve(Light)%Mean_Count)**2.0
          Light_Curve_total(Light)%Variance=Light_Curve_total(Light)%Variance &
              + (Net_total(Light,i) - Light_Curve_total(Light)%Mean_Count)**2.0
       end do
       Light_Curve(Light)%Variance = Light_Curve(Light)%Variance / real(Nbin-1)
       Light_Curve_total(Light)%Variance = Light_Curve_total(Light)%Variance /&
                                                                real(Nbin-1)

       call chi_square_test( Net_total(Light,:), Nbin, &
            Light_Curve_total(Light) )
       call kolmogorov_smirnov_test( Net_total(Light,:), Nbin, &
            Light_Curve_total(Light) )

       call chi_square_test( Net_Count(Light,:), Nbin, &
            Light_Curve(Light) )
       call kolmogorov_smirnov_test( Net_Count(Light,:), Nbin, &
            Light_Curve(Light) )

       call pgsvp(0.0 , 1.0 , 1.0-(Light+0.5)*Height , 1.0-(Light-0.5)*Height)
       call pgsch(4. * Character_Size) ; call pgsci(2)
       call pgmtxt('T' , -1. , 0.5 , 0.5 ,'RATES  '//achar(iachar("0")+Light))
       call pgsch(2.*Character_Size) ; call pgsci(5)
       call pgmtxt('T' , -5. , 0.10 , 0.0 ,   'Mean  c/s                  :')
       call pgmtxt('T' , -7. , 0.10 , 0.0 ,   'Variance (c/s)**2          :')

       if (trim(Which_Tests) == 'both' .or. trim(Which_Tests) == 'chi') then
         call pgmtxt('T' , -9. , 0.10 , 0.0 ,   'Chi Square (Total)         :')
       end if

       if (trim(Which_Tests) == 'both' .or. trim(Which_Tests) == 'ks') then
         call pgmtxt('T' , -12.5 , 0.10 , 0.0 , 'Kolmogorov_Smirnov (Total) :')
       end if
 
       call pgsci(1)

       call string_of_real(Light_Curve(Light)%Mean_Count/Bin_size, String, Length)
       call string_of_real(Light_Curve_total(Light)%Mean_Count/Bin_size, String_t, Length_t)
       call pgmtxt('T', -5., 0.5, 0.0, String_t(1:Length_t)//' Net: '//String(1:Length))

       call string_of_real(Light_Curve(Light)%Variance/(Bin_size**2.0), String, Length)
       call string_of_real(Light_Curve_total(Light)%Variance/(Bin_size**2.0), String_t, Length_t)
       call pgmtxt('T', -7., 0.5, 0.0, String_t(1:Length_t)//' Net: '//String(1:Length))

       call string_of_real(real(Nbin-1), String, Length)

       if (trim(Which_Tests) == 'both' .or. trim(Which_Tests) == 'chi') then
         call string_of_real(Light_Curve_total(Light)%Chi_Square%Stat, String_t , Length_t)
         call pgmtxt('T', -9., 0.5, 0.0, String_t(1:Length_t)//' ('//String(1:Length)//' dof.)')

         call string_of_real(Light_Curve_total(Light)%Chi_Square%Prob, String_t, Length_t)
         call pgmtxt('T', -10.5, 0.5, 0.0, 'Prob. = '//String_t(1:Length_t))
       end if


       if (trim(Which_Tests) == 'both' .or. trim(Which_Tests) == 'ks') then
         call string_of_real(Light_Curve_total(Light)%K_S%Stat, String_t, Length_t)
         call pgmtxt('T', -12.5, 0.5, 0.0, String_t(1:Length_t))

         call string_of_real(Light_Curve_total(Light)%K_S%Prob, String_t, Length_t)
         call pgmtxt('T', -14. , 0.5, 0.0, 'Prob. = '//String_t(1:Length_t))
       end if 
    end do

  end subroutine variability_tests

  !-------------------------------------------------------------------------------------
  subroutine chi_square_test(Count , Nbin , Lcurve)
  !-------------------------------------------------------------------------------------
  ! Returns the chi square statistic and probability of the hypothesis non variable
  ! source. So, the Degree of freedom (Df) is equal to Nbin-1.

    real(kind=single),dimension(:) :: Count
    integer,intent(in) :: Nbin
    type(data_tests),intent(inout) :: Lcurve

    real(kind=single) :: Stat 
    integer :: Df , i

    Df = Nbin - 1
    Stat = 0.
    do i=1,Nbin
       Stat = Stat + ( Count(i) - Lcurve%Mean_Count )**2
    end do
    Stat = Stat / Lcurve%Mean_Count
    Lcurve%Chi_Square%Stat = Stat
    Lcurve%Chi_Square%Prob = gamma_function_Q(0.5*Df , 0.5*Stat)

  end subroutine chi_square_test

  !-------------------------------------------------------------------------------------
  subroutine kolmogorov_smirnov_test(Count , Nbin , Lcurve)
  !-------------------------------------------------------------------------------------
  ! Compares the cumulative rate distribution with the cumulative expected poisson
  ! distribution. Returns the corresponding K_S statistic D, and the signifiance level
  ! Prob. Great values of Prob significate there is no variability in the input
  ! distribution.

    real(kind=single),dimension(:) :: Count
    integer,intent(in) :: Nbin
    type(data_tests),intent(inout) :: Lcurve

    real(kind=single) :: D, en, Nbin_Real, ir, Cumulative_Poisson_Value, fn, f0, KSprob
    integer :: i, j

    D = 0.0;  f0=0.0
    call sort(Count)
    Nbin_Real = real(Nbin)
    en=Nbin_Real
    
!!    en = (sqrt((Nbin_Real*Nbin_Real)/(Nbin_Real+Nbin_Real)))
!!    en = sqrt(Nbin_Real)
    ir = 0.
    do j=1,Nbin_Real
        fn=j/en  
        ir=amax1(abs(f0-Count(j)),abs(fn-Count(j)))
        if (ir > D) D=ir
        f0=fn
    end do
   Lcurve%K_S%Stat = D
!   print*,sqrt(en)*D
   Lcurve%K_S%Prob = kolmogorov_smirnov_prob(sqrt(en)*D)
   KSprob = Lcurve%K_S%Prob
!    D = 0.
!    call sort(Count)
!    Nbin_Real = real(Nbin)
!    en = sqrt(Nbin_Real)
!    ir = 0.
!    do i=1,Nbin
!       Cumulative_Poisson_Value = 0.5*( 1. - Gauss_Integral_Function( &
!            (Lcurve%Mean_Count-Count(i)) / sqrt(2.*Lcurve%Mean_Count) ) )
!       D = max( D , abs(ir/Nbin_Real - Cumulative_Poisson_Value ) )
!       ir = real(i)
!       D = max( D , abs(ir/Nbin_Real - Cumulative_Poisson_Value) )
!    end do
!    Lcurve%K_S%Stat = D
!    Lcurve%K_S%Prob = kolmogorov_smirnov_prob((en+0.12+0.11/en) * D)

  end subroutine kolmogorov_smirnov_test

  !-------------------------------------------------------------------------------------
  ! Many numerical functions or subroutines used by previous algorithms of variability
  ! are farther implemented. They might be icluded in a library if other SAS task want
  ! to use them (Gauss_Integral_Function, gamma_function_P, gamma_function_Q,
  ! series_gamma_func, fraction_gamma_function, gamma_log).
  !-------------------------------------------------------------------------------------

  !-------------------------------------------------------------------------------------
  function Gauss_Integral_Function(x)
  !-------------------------------------------------------------------------------------
  ! Returns the error function Gauss_Integral_Function(x).

    real(kind=single),intent(in) :: x
    real(kind=single) :: Gauss_Integral_Function

    if (x < 0.) then
       Gauss_Integral_Function = -gamma_function_P(0.5 , x**2.0)
    else
       Gauss_Integral_Function = gamma_function_P(0.5 , x**2.0)
    end if
  end function Gauss_Integral_Function

  !-------------------------------------------------------------------------------------
  function gamma_function_P(a,x)
  !-------------------------------------------------------------------------------------
  ! Returns the incomplete gamma function P(a,x) for a > 0. and x >= 0.

    real(kind=single),intent(in) :: a , x
    real(kind=single) :: gamma_function_P , gammcf , gamser , gln

    if (x < a+1.) then
       call series_gamma_func(gamser,a,x,gln)       ! Uses the serie representation
       gamma_function_P = gamser
    else
       call Fraction_gamma_function(gammcf,a,x,gln) ! Uses the complement of the
       gamma_function_P = 1. - gammcf               ! continued fraction representation
    end if
  end function gamma_function_P

  !-------------------------------------------------------------------------------------
  function gamma_function_Q(a,x)
  !-------------------------------------------------------------------------------------
  ! Returns the incomplete gamma function Q(a,x)=1-P(a,x) for a > 0. and x >= 0.

    real(kind=single),intent(in) :: a , x
    real(kind=single) :: gamma_function_Q , gammcf , gamser , gln

    if (x < a+1.) then
       call series_gamma_func(gamser,a,x,gln)         ! Uses the complement of the serie
       gamma_function_Q = 1. - gamser                 ! representation
    else
       call Fraction_gamma_function(gammcf,a,x,gln)   ! Uses the continued fraction
       gamma_function_Q = gammcf                      ! representation
    end if
  end function gamma_function_Q

  !-------------------------------------------------------------------------------------
  subroutine series_gamma_func(gamser ,a , x , gln)
  !-------------------------------------------------------------------------------------
  ! Returns the incomplete gamma function P(a,x) evaluated by its series representation
  ! as gamser. Also returns Ln(gamma(a) as gln. Warning : this routine works only if 
  ! a > 0. and x >= 0.

    real(kind=single),intent(in) :: a , x
    real(kind=single),intent(out) :: gamser , gln
    real(kind=single),parameter :: eps = 3.e-7
    integer,parameter :: imax = 100
    integer :: i
    real(kind=single) :: del , sum

    gln = gamma_log(a)
    if ( x == 0. ) then
       gamser = 0.
    else
       del = 1./a
       sum = del
       i = 1
       do while ( (i <= imax) .and. (abs(del) >= abs(sum)*eps) )
          del = del * x/(a+i)
          sum = sum + del
          i = i + 1
       end do
       gamser = sum * exp(-x + a*log(x) - gln)
    end if
  end subroutine series_gamma_func

  !-------------------------------------------------------------------------------------
  subroutine fraction_gamma_function(gammcf ,a , x , gln)
  !-------------------------------------------------------------------------------------
  ! Returns the incomplete gamma function Q(a,x) evaluated by its continued fraction
  ! representation as gammcf. Also returns Ln(gamma(a)) as gln. Warning : this routine
  ! works only if x >0. and a > 0.

    real(kind=single),intent(in) :: a , x
    real(kind=single),intent(out) :: gammcf , gln
    real(kind=single),parameter :: eps = 3.e-7 , min_float = 1.e-30
    integer,parameter :: imax = 100
    integer :: i
    real(kind=single) :: an , b , c , d , del , h

    gln = gamma_log(a)
    b = x + 1. - a
    c = 1./min_float
    d = 1./b
    h = d
    i = 1
    del = 2.                     ! Non significative value use for first loop iteration.
    do while ( (i <= imax) .and. (abs(del-1.) >= eps) )
       an = -i*(i-a)
       b = b + 2.
       d = an*d + b
       if ( abs(d) < min_float ) d = min_float
       c = b + an/c
       if ( abs(c) < min_float ) c = min_float
       d = 1./d
       del = d * c
       h = h * del
    end do
    gammcf = exp(-x + a*log(x) - gln) * h
  end subroutine fraction_gamma_function

  !-------------------------------------------------------------------------------------
  function gamma_log(xx)
  !-------------------------------------------------------------------------------------
  ! Returns Lanczos approximation of the value Ln(gamma(xx)) for xx > 0.

    real(kind=single),intent(in) :: xx
    real(kind=single) :: gamma_log
    real(kind=double) :: ser
    real(kind=double),dimension(6) :: cof = (/ 76.18009172947146d0 , &
         -86.50532032941677d0 , 24.01409824083091d0 , -1.231739572450155d0 , &
         .1208650973866179d-2 , -0.5395239384953d-5 /)
    real(kind=double) :: stp = 2.5066282746310005d0
    real(kind=double) :: x , y , tmp
    integer :: j

    ser = 1.000000000190015d0
    x = xx
    y = x
    tmp = x + 5.5d0
    tmp = (x+0.5d0) * log(tmp) - tmp
    do j=1,6
       y = y + 1.0d0
       ser =  ser + cof(j)/y
    end do
    gamma_log = real(tmp + log(stp * ser/x))
  end function gamma_log

  !-------------------------------------------------------------------------------------
  recursive subroutine sort(arr)
  !-------------------------------------------------------------------------------------
  ! Sorts an array arr(1:n) into ascending numerical order. arr is replaced on output
  ! by its sorted rearrangement

    real(kind=single),dimension(:),intent(inout) :: arr
    integer :: begin , end
    real(kind=single) :: elt , tmp
    integer :: i , j , med

    begin = 1 ; end = size(arr)
    if (end > begin) then
       elt = arr(begin)
       i = begin + 1 ; j = end
       do while (i < j)
          do while ( (arr(i) < elt) .and. (i < j) )
             i = i + 1 ; arr(i) = arr(i)
          end do
          do while ( (arr(j) >= elt) .and. (j > i) )
             j = j - 1 ; arr(j) = arr(j)
          end do
          if (i < j) then
             tmp = arr(i)
             arr(i) = arr(j)
             arr(j) = tmp
          end if
       end do
       if (arr(i) < elt) then
          med = i
       else
          med = i - 1
       end if
       tmp = arr(begin)
       arr(begin) = arr(med)
       arr(med) = tmp
       if (med-begin > 1) call sort(arr(begin : med-1))
       if (end-med > 1) call sort(arr(med+1 : end))
    end if
  end subroutine sort

  !-------------------------------------------------------------------------------------
  function kolmogorov_smirnov_prob(lambda)
  !-------------------------------------------------------------------------------------
  ! Kolmogorov-Smirnov probability function.

    real(kind=single) :: kolmogorov_smirnov_prob
    real(kind=single),intent(in) :: lambda
    real(kind=single),parameter :: eps1=0.001 , eps2=1.e-8
    integer :: j
    real(kind=single) :: lambda2 , term , last_term , fac

    lambda2 = -2.*lambda**2.0
    fac = 2.
    j = 1
    last_term = 0.
    term = fac*exp(lambda2*j**2.0)
    kolmogorov_smirnov_prob = term
    do while ( (abs(term) > eps1*last_term) .and. &
         (abs(term) > eps2*kolmogorov_smirnov_prob) .and. (j < 100) )
       last_term = abs(term)
       j = j + 1
       fac = -fac
       term = fac*exp(lambda2*j**2.0)
       kolmogorov_smirnov_prob = kolmogorov_smirnov_prob + term
    end do
    if (j >= 100) kolmogorov_smirnov_prob = 1.
  end function kolmogorov_smirnov_prob

  !-------------------------------------------------------------------------------------
  function belong(String , String_Table)
  !-------------------------------------------------------------------------------------
  ! Returns a boolean telling if a string belongs to a string table.

    logical :: belong
    character(len=*),intent(in) :: String
    character(len=*),dimension(:),intent(in) :: String_Table
    integer :: i

    belong = .false.
    do i=1,size(String_Table)
       belong = belong .or. ( String == String_Table(i) )
    end do

  end function belong

  !-------------------------------------------------------------------------------------
  subroutine string_of_real(Nb , String , Length)
  !-------------------------------------------------------------------------------------
  ! Converts real numbers into plottable character chain. Gives also its length.

    real(kind=single),intent(in) :: Nb
    character(len=20),intent(out) :: String
    integer,intent(out) :: Length
    integer :: MM , puiss

!    puiss = max( 0 , min( 6 , 6-int(log10(Nb)) ) )
    puiss = max( 0 , min( 8 , 8-int(log10(Nb)) ) )
    MM = nint(Nb*(10**puiss))
    call pgnumb(MM , -puiss , 0 , String , Length)

  end subroutine string_of_real

  !-------------------------------------------------------------------------------------
  subroutine string_of_real2(Nb , String , Length)
  !-------------------------------------------------------------------------------------
  ! Converts real numbers into plottable character chain. Gives also its length.

    real(kind=double),intent(in) :: Nb
    character(len=30),intent(out) :: String
    integer,intent(out) :: Length
    integer :: MM , puiss

    puiss = max( 0 , min( 8 , 8-int(log10(Nb)) ) )
    MM = nint(Nb*(10**puiss))
    call pgnumb(MM , -puiss , 0 , String , Length)

  end subroutine string_of_real2

  !-------------------------------------------------------------------------------------
  function change_case(instring,dir) result(outstring)
  !-------------------------------------------------------------------------------------
  ! Switch from lower to upper case or vice-versa: From Jean Ballet
  ! instring : input string
  ! dir      : direction (1: to lower case; 2: to uppercase; 0: switch case)

    character(len=*), intent(in) :: instring
    integer, intent(in) :: dir
    character(len=len(instring)) :: outstring

  ! ASCII positions of lower case and upper case letters

    integer, parameter :: start_up  = iachar("A"), end_up  = iachar("Z"), &
                          start_low = iachar("a"), end_low = iachar("z"), &
                          offset = start_low - start_up

    integer :: length, icur, pos

    length = len(instring)
    outstring = instring
    do icur = 1, length
       pos = iachar(instring(icur:icur))
       if (pos >= start_up .and. pos <= end_up .and. dir < 2) then
          pos = pos + offset
       else if (pos >= start_low .and. pos <= end_low .and. dir /= 1) then
          pos = pos - offset
       endif
       outstring(icur:icur) = achar(pos)
    enddo

  end function change_case  
  
end module lcplot_mod
